package File::Find::Match::Sugar;
use strict;
use warnings;
use Carp;

croak <<__DEPRECATED__;
File::Find::Match::Sugar is DEPRECATED.
File::Find::Match went under an API change between versions 0.06
and 0.08. Appologies, but it was unavoidable. It is still not 1.0 yet,
so this should be forgivable.
__DEPRECATED__
